#include "GeneralInfo.h"
#include <string>
#include <stdlib.h>
#include <stdio.h>

// x processor usages
// used memory
// page faults
// used sems
// used ports
// used threads
// used teams
const int numAdditionalInfos = 6;

// Constructor, sets up the infos and prepares the usage measuring
ASysMonInfo::ASysMonInfo() : MasterSysMonInfo() {
	get_system_info(&sysInfo);
	numberInfos = sysInfo.cpu_count + numAdditionalInfos;
	theInfo = new SysMonInfo[numberInfos];
	int x;
	// Processor Usage section
	processorTime = new bigtime_t[sysInfo.cpu_count];
	previousTime = new bigtime_t[sysInfo.cpu_count];

	for (x = 0; x < sysInfo.cpu_count; ++x) {
		// Set names
		if (sysInfo.cpu_count > 1)
			theInfo[x].name = "Processor " + numtostr(x+1) + " usage";
		else
			theInfo[x].name = "Processor usage";
		// Set category
		theInfo[x].category = "Processor usage";
		// Set units
		theInfo[x].units = "%";
		// Set hasTopValue
		theInfo[x].hasTopValue = true;
		// Set hasBottomValue
		theInfo[x].hasBottomValue = true;
		// Set topValue
		theInfo[x].topValue = 100;
		// Set bottomValue
		theInfo[x].bottomValue = 0;
		// Set defaultInterval
		theInfo[x].defaultInterval = 0.2;
		// Get the current active time for each processor
		processorTime[x] = sysInfo.cpu_infos[x].active_time;
		previousTime[x] = system_time();
	}

	// used memory
	theInfo[x].name = "Used memory";
	theInfo[x].category = "General Info";
	theInfo[x].units = "MB";
	theInfo[x].hasTopValue = true;
	theInfo[x].hasBottomValue = true;
	theInfo[x].topValue = sysInfo.max_pages / 256;
	theInfo[x].bottomValue = 0;
	theInfo[x].defaultInterval = 0.2;

	// page faults
	x++;
	theInfo[x].name = "Page Faults";
	theInfo[x].category = "General Info";
	theInfo[x].units = "Faults";
	theInfo[x].hasTopValue = false;
	theInfo[x].hasBottomValue = true;
	theInfo[x].topValue = 100; // not used
	theInfo[x].bottomValue = 0;
	theInfo[x].defaultInterval = 0.2;

	// used sems
	x++;
	theInfo[x].name = "Used Semaphores";
	theInfo[x].category = "General Info";
	theInfo[x].units = "Sems";
	theInfo[x].hasTopValue = true;
	theInfo[x].hasBottomValue = true;
	theInfo[x].topValue = sysInfo.max_sems;
	theInfo[x].bottomValue = 0;
	theInfo[x].defaultInterval = 0.2;
	
	// used ports
	x++;
	theInfo[x].name = "Used Ports";
	theInfo[x].category = "General Info";
	theInfo[x].units = "Ports";
	theInfo[x].hasTopValue = true;
	theInfo[x].hasBottomValue = true;
	theInfo[x].topValue = sysInfo.max_ports;
	theInfo[x].bottomValue = 0;
	theInfo[x].defaultInterval = 0.2;

	// used threads
	x++;
	theInfo[x].name = "Used Threads";
	theInfo[x].category = "General Info";
	theInfo[x].units = "Threads";
	theInfo[x].hasTopValue = true;
	theInfo[x].hasBottomValue = true;
	theInfo[x].topValue = sysInfo.max_threads;
	theInfo[x].bottomValue = 0;
	theInfo[x].defaultInterval = 0.2;

	// used teams
	x++;
	theInfo[x].name = "Used Teams";
	theInfo[x].category = "General Info";
	theInfo[x].units = "Teams";
	theInfo[x].hasTopValue = true;
	theInfo[x].hasBottomValue = true;
	theInfo[x].topValue = sysInfo.max_teams;
	theInfo[x].bottomValue = 0;
	theInfo[x].defaultInterval = 0.2;
}

// x processor usages
// used memory
// page faults
// used sems
// used ports
// used threads
// used teams
float ASysMonInfo::GetValue(int NumberInfo) {
	get_system_info(&sysInfo);
	if (NumberInfo >= 0 && NumberInfo < sysInfo.cpu_count) {
		currentTime = system_time();
		currentProcessorTime = sysInfo.cpu_infos[NumberInfo].active_time;
		float tempVal =  (float)(currentProcessorTime - processorTime[NumberInfo]) /
			             (float)(currentTime - previousTime[NumberInfo]) * 100;
		previousTime[NumberInfo] = currentTime;
		processorTime[NumberInfo] = currentProcessorTime;
		return tempVal;
	}
	else if (NumberInfo == sysInfo.cpu_count) {
		return sysInfo.used_pages / 256;
	}
	else if (NumberInfo == sysInfo.cpu_count + 1) {
		return sysInfo.page_faults;
	}
	else if (NumberInfo == sysInfo.cpu_count + 2) {
		return sysInfo.used_sems;
	}
	else if (NumberInfo == sysInfo.cpu_count + 3) {
		return sysInfo.used_ports;
	}
	else if (NumberInfo == sysInfo.cpu_count + 4) {
		return sysInfo.used_threads;
	}
	else if (NumberInfo == sysInfo.cpu_count + 5) {
		return sysInfo.used_teams;
	}
}

ASysMonInfo::~ASysMonInfo() {
	cout << "~ASysMonInfo()\n";
	delete [] theInfo;
	delete [] processorTime;
	delete [] previousTime;
}


MasterSysMonInfo* GetInfoClass() {
	return new ASysMonInfo;
}